# -*- coding: utf-8 -*-
#MoxRangeEditor 2021.12.01 MayaLT対応、play,stopをトグル化。setTimeLine機能追加、シーンに保存機能追加
import maya.cmds as cmds
import maya.mel as mm
from functools import partial
import re #正規表現モジュール

class MoxRangeEditor():
    def __init__(self):
        if cmds.window('MoxRangeEditor', exists=True):cmds.deleteUI('MoxRangeEditor')
            
        cmds.window('MoxRangeEditor', menuBarVisible=True, tlb=True, s=1, t="MoxRangeEditor", resizeToFitChildren=1, mnb=1, mxb=0)
        cmds.columnLayout('MainColumnLayout', adjustableColumn=True, rowSpacing=5, columnAttach=["both",5])
        
        #メニューバー
        cmds.menuBarLayout("menu")
        cmds.menu( label='Save' )
        cmds.menuItem( label='Save Preset',command="MoxRangeEditor.presetSave()",ann=u"presetを一時的にfileinfoに書き込みます。確定するにはシーンを保存する必要があります。")
        cmds.menuItem( label='Save ScenePreset',command=partial(self.presetSave,True), ann=u"presetをfileinfoに書き込みするとともに、シーンの保存を同時に実行します。")
        cmds.menuItem( label='Remove Preset',command = "cmds.fileInfo(remove='MoxRangeEditorPreset')", ann=u"presetをfileinfoから削除します。")
        
        cmds.button("add range", c="MoxRangeEditor.addRange()")
        cmds.showWindow('MoxRangeEditor')
        
        #シーン内fileinfoにデータが保存されていれば
        if (len(cmds.fileInfo("MoxRangeEditorPreset", q=1)) != 0):
            self.presetLoad()
        
    #formを追加する        
    def addRange(self):
        cmds.setParent( 'MoxRangeEditor|MainColumnLayout' ) #ここにいれるぜ！
        
        #Preset最大値の数字を取り出す
        if cmds.layout('MoxRangeEditor|MainColumnLayout',q=True, ca=True) != None: #すでに存在するなら
            lastprop = cmds.layout('MoxRangeEditor|MainColumnLayout',q=True, ca=True)[-1] #最後のRangeEditorを取得
            
            m = re.match('[^\d]*(\d+).*$', lastprop)	#\dは正規表現で任意の数字[0-9]
            if m:
                i = int( m.groups()[0] ) +1
            else:i=1
        else:i=1
        
        Presetnum  = '_'+str(i)
        PresetLay = cmds.formLayout(Presetnum)
        
        cmds.rowLayout(nc=6)
        cmds.textField("name"+Presetnum, w=100)
        cmds.intField("startF"+Presetnum, w=50)
        cmds.intField("endF"+Presetnum, w=50)
        cmds.button("Play"+Presetnum, l="play", c="MoxRangeEditor.rangePlay(%r)"%Presetnum, w=30)
        cmds.button("Set"+Presetnum, l="set",c="MoxRangeEditor.setTimeLine(%r)"%Presetnum)
        cmds.button("Del"+Presetnum, l="del",c="MoxRangeEditor.delForm(%r)"%Presetnum)
        cmds.setParent("..")
        cmds.setParent( '..' )

    #再生&停止する
    def rangePlay(self, Presetnum):
        lastprop = cmds.layout('MoxRangeEditor|MainColumnLayout',q=True, ca=True)[-1] #最後のRangeEditorを取得
        m = re.match('[^\d]*(\d+).*$', lastprop)
        rangeMaxNum = int( m.groups()[0] )
        
        #再生中なら停止、それ以外なら再生
        if(cmds.play(q=1, st=1)):
            cmds.play(state=False)
            for n in range(rangeMaxNum):
                cmds.button("Play_"+str(n+1), e=1, l="play")
        else:
            startF = cmds.intField("startF"+Presetnum, q=True, v=True)
            endF = cmds.intField("endF"+Presetnum, q=True, v=True)
            cmds.playbackOptions(min=startF, max=endF)
            cmds.play(state=True)
            for n in range(rangeMaxNum):
                cmds.button("Play_"+str(n+1), e=1, l="stop")

    #タイムラインをセットする
    def setTimeLine(self, Presetnum):
        startF = cmds.intField("startF"+Presetnum, q=True, v=True)
        endF = cmds.intField("endF"+Presetnum, q=True, v=True)
        cmds.playbackOptions(ast=startF, aet=endF)
        cmds.playbackOptions(min=startF, max=endF)


    #formを削除する
    def delForm(self, set):
        try:
            cmds.deleteUI('MoxRangeEditor|MainColumnLayout|'+set)
        except:pass
    
    #入力情報をノードに保存する　保存用
    def presetSave(self,save=True,*args):
        cmds.fileInfo("MoxRangeEditorPreset", MoxRangeEditor.getPresetParam())
        if(save):
            cmds.file(save=True)
       
            
    #入力情報を文字列で出力する  保存用
    def getPresetParam(self):
        if (cmds.layout('MoxRangeEditor|MainColumnLayout',q=True, numberOfChildren=True) == 1):
            return "";
        else:    
            lastprop = cmds.layout('MoxRangeEditor|MainColumnLayout',q=True, ca=True)[-1] #最後のRangeEditorを取得
            m = re.match('[^\d]*(\d+).*$', lastprop)
            rangeMaxNum = int( m.groups()[0] )
            result = ""
            
            for n in range(rangeMaxNum):
                result += (cmds.textField("name_"+str(n+1), q=1, tx=1) + ",")
                startF = cmds.intField("startF_"+str(n+1), q=1, v=1)
                result += (str(startF) + ",")
                endF = cmds.intField("endF_"+str(n+1), q=1, v=1)
                result += (str(endF) + ";")
            return result
        
        
    #プリセットの情報をUIとして復元する
    def presetLoad(self):
        strParam = cmds.fileInfo("MoxRangeEditorPreset", q=1)[0]
        if(strParam != ""):
            listParam = strParam.split(";")
            del listParam[-1]
            cmds.setParent( 'MoxRangeEditor|MainColumnLayout' ) #ここにいれるぜ！
            
            for i,param in enumerate(listParam):
                Presetnum  = '_'+str(i+1)
                PresetLay = cmds.formLayout(Presetnum)
                
                cmds.rowLayout(nc=6)
                cmds.textField("name"+Presetnum, w=100, tx= param.split(",")[0])
                cmds.intField("startF"+Presetnum, w=50, v= int( param.split(",")[1]))
                cmds.intField("endF"+Presetnum, w=50, v= int( param.split(",")[2]))
                cmds.button("Play"+Presetnum, l="play", c="MoxRangeEditor.rangePlay(%r)"%Presetnum, w=30)
                cmds.button("Set"+Presetnum, l="set",c="MoxRangeEditor.setTimeLine(%r)"%Presetnum)
                cmds.button("Del"+Presetnum, l="del",c="MoxRangeEditor.delForm(%r)"%Presetnum)
                cmds.setParent("..")
                cmds.setParent( '..' )

        
MoxRangeEditor = MoxRangeEditor()